package com.qima.gift.appcommon.space.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import com.qima.gift.appcommon.space.SpaceConst;
import com.qima.gift.appcommon.space.base.IFileSource;
import com.qima.gift.appcommon.space.base.SpaceFileReadFileSource;
import com.qima.gift.appcommon.space.bean.SpaceFile;
import com.qima.gift.appcommon.space.bean.StorageArea;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.utils.JsonUtils;
import io.swagger.v3.core.util.Json;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.InputStreamSource;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.Date;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 专门负责存储
 * @author King
 * @since 2023/6/13 11:40
 */
@Component
public class StorageService {

    @Value("${app.storage.path}")
    private String storagePathStr;
    private StorageArea defaultStorageArea;

    @PostConstruct
    protected void init(){
        defaultStorageArea = initStorageAreaDefault(storagePathStr);
        if(defaultStorageArea == null) {
            throw new AppException("初始化存储空间失败，path=" + storagePathStr);
        }
    }

    public void doStorage(SpaceFile file, IFileSource source) {
        File f = findStorageFile(defaultStorageArea, file);
        OutputStream fos = null;
        InputStream is = null;
        try{
            fos = new FileOutputStream(f);
            is = source.getInputStream();
            long size = IoUtil.copy(is, fos);
            file.setSize(size);
            file.setSaveSize(size);
            file.setSaveName(file.getId());
            file.setStorageArea(defaultStorageArea.getId());
            file.setStorageEncode(SpaceConst.SpaceFileStorageEncodeEnum.No.getKey());
        } catch (Exception e) {
            throw new AppException("保存文件失败",e).setM(JsonUtils.toJson(file));
        } finally {
            IoUtil.close(is);
            IoUtil.close(fos);
        }
    }

    public void deleteStorage(SpaceFile file) {
        File f = findStorageFile(defaultStorageArea, file);
        if(f.exists()) {
            f.delete();
        }
    }

    public IFileSource readStorage(SpaceFile file) {
        File f = findStorageFile(defaultStorageArea, file);
        if(f.exists()) {
            return new SpaceFileReadFileSource(file,f);
        } else {
            throw new AppException("文件不存在").setM(JsonUtils.toJson(file));
        }
    }

    protected static File findStorageFile(StorageArea area, SpaceFile file) {
        String yearMonth = DateUtil.format(file.getGmtCreate(),"yyyy_MM");
        File df = new File(area.getPath().getPath()+ CommonConst.FILE_SEPARATOR + yearMonth);
        if(!df.exists()) {
            df.mkdirs();
        }
        File f = new File(df.getPath()+CommonConst.FILE_SEPARATOR+file.getId());
        if(!f.exists()) {
            try{
                f.createNewFile();
            } catch (IOException e) {
                throw new AppException("文件创建失败",e);
            }
        }
        return f;
    }

    protected static StorageArea initStorageAreaDefault(String storagePathStr) {
        StorageArea r = new StorageArea();
        r.setId(SpaceConst.STORAGE_AREA_DEFAULT_ID);
        r.setAreaName(SpaceConst.STORAGE_AREA_DEFAULT_NAME);
        File f = new File(storagePathStr);
        if(!f.exists()) {
            if(f.mkdirs()) {
                r.setPath(f);
                return r;
            } else {
                return null;
            }
        } else {
            r.setPath(f);
            return r;
        }
    }

//    public static void main(String[] args) {
//        String path = "D:\\work\\storage";
//        StorageArea area = initStorageAreaDefault(path);
//        SpaceFile spaceFile = new SpaceFile();
//        spaceFile.setId("a11111-22222");
//        spaceFile.setGmtCreate(new Date());
//        File sf = findStorageFile(area, spaceFile);
//        System.out.println(sf.exists());
//    }

}
